;nyquist plug-in
;version 3
;type process
;categories "http://lv2plug.in/ns/lv2core/#UtilityPlugin"
;name "Transcription Pauses..."
;action "Inserting regular pauses..."

;; transcription-pauses.ny by Steve Daulton Dec 2013
;; Released under terms of the GNU General Public License version 2:
;; http://www.gnu.org/licenses/old-licenses/gpl-2.0.html 
;;
;; For information about writing and modifying Nyquist plug-ins:
;; http://wiki.audacityteam.org/wiki/Nyquist_Plug-ins_Reference

;control showhelp "Run effect or show help" choice "Run Effect,Show Help" 0 
;control interval "Interval (seconds)" real "" 30 0 50
;control sec-silence "Pause for (seconds)" real "" 25 0 50
;control overlap "Overlap (seconds)" real "" 2 0 10

(setq err "") ; Initialise error message.

(defun help ()
  (format nil
"Transcription Pauses:~%
This effect inserts silences at regular intervals into
the selected audio.~%
CONTROLS:~%
Interval: Silence will be inserted after each interval.~%
Pause for: How much silence will be inserted each time.~%
Overlap: When set greater than 0 seconds, the audio will
resume after each silence taking a starting point from
before the end of the previous section. Thus the end of
one audio section will be repeated at the start of the
next audio section. 'Overlap' sets how much audio to
repeat."))

(defun errorcheck ()
  (cond
    ((= interval overlap)(setq err
      (format nil "The Interval and overlap are equal.~%~
                  This will cause the same section to be repeated~%~
                  forever, which is not allowed.~%")))
    ((> overlap interval)(setq err
      (format nil "Audio sections cannot be overlapped by more than.~%~
                  the duration of the section.~%~
                  Reduce the Overlap amount or increase the Interval.~%")))
    ((> overlap (* interval 0.9))(setq err
      (format nil "Overlapping by more than 90% is not allowed.~%")))
    ((>= interval (get-duration 1))(setq err
      (format nil "Interval set longer than selection length.~%")))
    (T nil)))

(defun spread (sig adur sildur olap)
  (let* ((dur (get-duration 1))
         (sections (/ (- dur olap) adur))
         (audiolen (+ (* (- sections 2)(+ adur olap))(* 2 adur)))
         (sillen (* (1- sections) sildur))
         (new-dur (+ audiolen sillen)))
    (do ((new-s (abs-env (s-rest new-dur)))
         (pos 0 (+ pos adur olap sildur))
         (start 0 (+ start adur))
         (end (+ adur olap) (+ end adur)))
        ((>= start dur) new-s)
      (setf new-s
        (sim new-s
          (at-abs pos (cue (extract-abs start end sig))))))))

(cond
  ((= showhelp 1)(format t "~a" (help))(help))
  ((errorcheck)(format nil "ERROR.~%~%~a" err))
  (T (multichan-expand #'spread s 
                                (float (- interval overlap))
                                (float sec-silence)
                                (float overlap))))